﻿// ***********************************************************************************
// SallyUIscale4K
// ***********************************************************************************
// Created by Sally Whitemane on VAM Hub
// Licensed under CC0 - https://creativecommons.org/publicdomain/zero/1.0/

// V8 changes
// * added UI offset and bar position to saved plugin data & removed debug checkbox with no longer relevant warning text
// * added button to deleted save plugin data
// 
// V7 changes
// * added button to save the current UI Scale as default, the scale should then be restored whenever the Plugin activates
//   It's a float value stored in Saves\PluginData\SallyUIscale4K\default.json
//   It assumes 3840x2160 as 1.0.
// * removed the "F1=Hide" text from lower left corner, there was not enough text space to show FPS above 99
//
// V6 changes
// * replaced bottom left toolbar text with field of view and frames per second
// * increased the original VAM UI scale slider maximum from 2x to 4x to allow 8K UI with original VAM scaling
//   Plugin must be in enabled/loaded, but deactivated state for this, removing the Plugin will reset this to the default 2x
// * activated hidden "Toggle Monitor"-mode button on toolbar, the button will stay visible when removing the Plugin will

using MVR.FileManagementSecure;
using SimpleJSON;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Text;
using UnityEngine;
using UnityEngine.UI;

namespace Sally { 
	public class UIscale4K : MVRScript {
        private const string SAVE_FOLDER = "Saves\\PluginData\\SallyUIscale4K";
		private const string SAVE_FILENAME = "default";
        private const string SAVE_EXTENSION = "json";
        private const int MIN_ACTIVE_HEIGHT = 1440;
		private const float SCALE_MIN = 0.5f;
		private const float SCALE_MAX = 4.0f;
		private const float SCALE_STEP = 0.05f;
		private const float FOV_STEP = 2.0f;
		private readonly string RES_PAGE_UP = Display.main.systemWidth + "x" + Display.main.systemHeight;
		private readonly string RES_PAGE_DOWN = GetLowResByNativeAR();

        private float autoUIscale = Screen.height / 2160f;
		private JSONStorableBool jbActive;
		private int newWidth = Screen.width;
		private int newHeight = Screen.height;
		private float aspectRatio = (float)Screen.width / (float)Screen.height;
		private Vector3 pos;
		private const string VERSION = "8";

		private readonly static bool debugRes = false;
		private readonly static int debugNatWidth = 2560;
		private readonly static int debugNatHeight = 1440;

		protected JSONStorableString jstringCurrentInfo;
		protected UIDynamicTextField uiTextWarn;
		protected JSONStorableFloat jfloatX;
		protected JSONStorableFloat jfloatY;
		protected JSONStorableFloat jfloatZ;
		protected JSONStorableFloat jfloatO;
		protected UIDynamicSlider uiSliderX;
		protected UIDynamicSlider uiSliderY;
		protected UIDynamicSlider uiSliderZ;
		protected UIDynamicSlider uiSliderO;

		private Text barText;

		void Start()
		{
			try
			{
				stopWatch.Start();
				// SceneAtoms (root)
				// +-> CoreControl
				//     +-> WorldScaleAdjust
				//         +->MainWindowUI
				//            +->MonitorModeUI
				//                +->Panel
				//                    +->Text     // <- text to change
				Transform MainWindowUI = SuperController.singleton.MonitorModeAuxUI.gameObject.transform.parent;
				Transform panel = MainWindowUI.GetChild(0);
				barText = panel.GetComponentInChildren<Text>();
				barText.fontSize = 26;
				barText.fontStyle = FontStyle.Bold;
				Transform ToggleMonitorButton = panel.GetChild(1);
				ToggleMonitorButton.gameObject.SetActive(true);

				// LEFT SIDE
				if (Screen.height < MIN_ACTIVE_HEIGHT)
				{
					jbActive = new JSONStorableBool("Active", false);
					SuperController.LogMessage("SallyUIscale4K NOT ACTIVATED to prevent unreachable UI (resolution height is < " + MIN_ACTIVE_HEIGHT + ")");
				}
				else
				{
					jbActive = new JSONStorableBool("Active", true);
				}
				CreateToggle(jbActive);
				jbActive.setCallbackFunction += ToggleActive;
				UIDynamicTextField uiText = CreateTextField(new JSONStorableString("Description", "<b>SallyUIscale4K</b> Version "+VERSION+ "\n\n>> Fix parts of the broken UI in high resolution desktop mode\n>> Allow UI scaling up to 8K\n\nTo auto-load this with VAM:\n1) Load it as [Session Plugin]\n2) go to [Session Plugin Presets]\n > [Change User Defaults...]\n > [Set Current As User Defaults]\n\n\n<b>Keys:</b>\nControl + Home = Toggle active\nControl + left/right = UI scale\nControl + up/down = FoV (Field of View)\nControl + page up = set " + RES_PAGE_UP + "\nControl + page down = set " + RES_PAGE_DOWN + "\n\n\nCreated by Sally Whitemane\npatreon.com/sallyw\nLicensed under CC0"));
				uiText.height = 1000f;

				// RIGHT SIDE
				CreateButton("Save as my default", true).button.onClick.AddListener(SavePluginDataJSON);
                CreateButton("Delete my default", true).button.onClick.AddListener(DeletePluginDataJSON);
                List<string> choices = new List<string>();
				foreach (Resolution r in Screen.resolutions)
				{
					// ignore low resolutions - minimum 1280x720
					if (r.width >= 1280 && r.height >= 720)
					{
						string res = r.width + "x" + r.height;
						// I get the same resolutions from Unity's Screen.resolutions multiple times
						// could be because I have multiple GPUs? -> remove doubles
						if (!choices.Contains(res))
							choices.Add(res);
					}
				}
				CreateScrollablePopup(new JSONStorableStringChooser("ResChooser", choices, Screen.width + "x" + Screen.height, "Resolution", PickRes), true).labelWidth = 150f;
				CreateButton("Apply resolution (click twice)", true).button.onClick.AddListener(ApplyRes);
				jstringCurrentInfo = new JSONStorableString("curRes", "");
				UIDynamicTextField dtextCurrentInfo = CreateTextField(jstringCurrentInfo, true);
				dtextCurrentInfo.height = 200f;

                jfloatX = new JSONStorableFloat("Bar X position", 0f, SetBarPositionX, -5f, 5f, true);
				uiSliderX = CreateSlider(jfloatX,true);

				jfloatY = new JSONStorableFloat("Bar Y position", 0f, SetBarPositionY, -5f, 5f, true);
				uiSliderY = CreateSlider(jfloatY, true);

				jfloatZ = new JSONStorableFloat("Bar Z position", -10f, SetBarPositionZ, -10f, 0f, true);
				uiSliderZ = CreateSlider(jfloatZ, true);

				jfloatO = new JSONStorableFloat("Offset", 0f, SetBarOffset, -300f, 300f, true);
				uiSliderO = CreateSlider(jfloatO, true);

				if (jbActive.val)
					Activate();
				UpdateCurrentText();
			}
			catch (Exception e)
			{
				SuperController.LogError("Exception: " + e);
			}
		}

		private static Stopwatch stopWatch = new Stopwatch();
		private static int fps = 1;
		private static int frame = 0;
		private static readonly string fpsHiRGB = "000";
		private static readonly string fpsLoRGB = "F00";
		private static string fpsRGB = fpsHiRGB;

		private static StringBuilder sbBar = new StringBuilder(80,80);

		void Update()
		{
			frame++;
			if (frame >= 30)
            {
				//SuperController.LogMessage("elapsed: ms - ticks - sec: " + stopWatch.ElapsedMilliseconds + " - " + stopWatch.ElapsedTicks + " - " + (1.0f / Stopwatch.Frequency) * stopWatch.ElapsedTicks);
				fps = (int)(0.5f + frame * (1.0f / ((1.0f / Stopwatch.Frequency) * stopWatch.ElapsedTicks)));
				stopWatch.Reset();
				stopWatch.Start();
				frame = 0;
				fpsRGB = fpsHiRGB;
				if (fps < 58.0f)
					fpsRGB = fpsLoRGB;

				sbBar.Length = 0;
				sbBar.Append("FoV ").Append(SuperController.singleton.monitorCameraFOV);
				// show UI scale? (not enough space if FPS number is to high/long)
				//sbBar.Append(" UI ").Append(SuperController.singleton.monitorUIScale.ToString("0.00"));
				sbBar.Append("\nFPS <color=#").Append(fpsRGB).Append(">").Append(fps).Append("</color>");
				barText.text = sbBar.ToString();
				// alt code to avoid GC - not working due to Security restrictions
				//barText.text = (string)sbBar.GetType().GetField("m_StringValue", BindingFlags.NonPublic | BindingFlags.Instance).GetValue(sbBar);
			}

			if (Input.GetKey("left ctrl") || Input.GetKey("right ctrl"))
			{
				// Toggle Active
				if (Input.GetKeyDown("home"))
				{
					ToggleActive(!jbActive.val);
					jbActive.val = !jbActive.val;
				}

				// FOV
				if (Input.GetKeyDown("up"))
				{
					SuperController.singleton.monitorCameraFOV -= FOV_STEP;
					if (SuperController.singleton.monitorCameraFOV < 20f)
						SuperController.singleton.monitorCameraFOV = 20f;
					
				}
				if (Input.GetKeyDown("down"))
				{
					SuperController.singleton.monitorCameraFOV += FOV_STEP;
					if (SuperController.singleton.monitorCameraFOV > 100f)
						SuperController.singleton.monitorCameraFOV = 100f;
				}

				// UI Scale
				if (Input.GetKeyDown("right"))
				{
					SuperController.singleton.monitorUIScale += SCALE_STEP;
					if (SuperController.singleton.monitorUIScale > SCALE_MAX)
						SuperController.singleton.monitorUIScale = SCALE_MAX;
				}
				if (Input.GetKeyDown("left"))
				{
					SuperController.singleton.monitorUIScale -= SCALE_STEP;
					if (SuperController.singleton.monitorUIScale < SCALE_MIN)
						SuperController.singleton.monitorUIScale = SCALE_MIN;
				}

				// Set native monitor resolution
				if (Input.GetKeyDown("page up"))
				{
					PickRes(RES_PAGE_UP);
					ApplyRes();
				}

				// Set low 1080p-like resolution
				if (Input.GetKeyDown("page down"))
				{
					PickRes(RES_PAGE_DOWN);
					ApplyRes();
				}

				// Set debug resolution
				if (debugRes && Input.GetKeyDown("end"))
				{
					PickRes(debugNatWidth+"x"+ debugNatHeight);
					ApplyRes();
				}
			}
		}

		void OnDestroy()
        {
			Deactivate();
			// reset to VAM default values
			SuperController.singleton.monitorUIScaleSlider.maxValue = 2.0f;
			//ToggleMonitorButton.gameObject.SetActive(false); // refuses to deaktivate
			barText.fontSize = 20;
			barText.fontStyle = FontStyle.Normal;
			barText.text = "Use F1 Key To\nToggle This Bar";
		}

		private void ToggleActive(bool b)
        {
			if (b)
			{
				if (Screen.height < MIN_ACTIVE_HEIGHT)
                {
					jbActive.SetVal(false);
					SuperController.LogMessage("SallyUIscale4K NOT ACTIVATED to prevent unreachable UI (resolution height is < " + MIN_ACTIVE_HEIGHT + ")");
					return;
                }
				Activate();
			}
			else
				Deactivate();
        }

		private void Activate()
        {
			bool restoreSaved = false;
            if (LoadPluginDataJSON())
            {
                // use saved JSON data
                restoreSaved = true;
                SuperController.LogMessage("SallyUIscale4K successfully loaded user default");
            }

            // 'fixedMonitorUIScale = 1f' assumes 1920x1080 I guess and therefore 2f makes 3840x2160 the new default
            // With 'monitorUIScale' untouched this leaves room to scale up to 8K.
            SuperController.singleton.fixedMonitorUIScale = 2f;
            SuperController.singleton.monitorUIScale = autoUIscale;
            SuperController.singleton.monitorUIScaleSlider.maxValue = SCALE_MAX;
			if (!restoreSaved)
			{
				SuperController.singleton.monitorUIYOffset = 30f;

				// for 3840x2160:
				pos.x = -4.36f;
				pos.y = 2.45f;
				pos.z = -4.25f;

				float aspectRatio = (float)Screen.width / (float)Screen.height;
				if (aspectRatio <= 1.2501f) // 5:4
				{
					// pos.x = -3.06f; // y = 2.45f - z = -4.25f
					// tested: 1800x1440, 2700x2160
					pos.x = -0.36f;
					pos.y = 0.29f;
					pos.z = -8.0f;
					SuperController.singleton.monitorUIYOffset = 0f;
					SuperController.LogMessage("SallyUIscale4K WARNING - Untested unsupported resolution.");
				}
				else if (aspectRatio <= 1.3334f) // 4:3
				{
					//pos.x = -3.27f; // y = 2.45f - z = -4.25f
					if (Screen.height == 1440) // 1920x1440
					{
						pos.x = -0.96f;
						pos.y = 0.72f;
						pos.z = -7.25f;
						SuperController.singleton.monitorUIYOffset = 0f;
					}
					else if (Screen.height == 1536) // 2048x1536
					{
						pos.x = -1.5f;
						pos.y = 1.12f;
						pos.z = -6.55f;
						SuperController.singleton.monitorUIYOffset = 4f;
					}
					else if (Screen.height == 2160) // 2880x2160
					{
						pos.x = -3.27f;
						pos.y = 2.45f;
						pos.z = -4.25f;
						SuperController.singleton.monitorUIYOffset = 30f;
					}
					else
					{
						SuperController.singleton.monitorUIYOffset = 0f;
						SuperController.LogMessage("SallyUIscale4K WARNING - Untested unsupported resolution.");
					}
				}
				else if (aspectRatio <= 1.5001f) // 3:2
				{
					//pos.x = -3.68f; // y = 2.45f - z = -4.25f
					if (Screen.height == 1440) // 2160x1440
					{
						pos.x = -1.08f;
						pos.y = 0.72f;
						pos.z = -7.25f;
						SuperController.singleton.monitorUIYOffset = 0f;
					}
					else if (Screen.height == 1504) // 2256x1504
					{
						pos.x = -1.47f;
						pos.y = 0.97f;
						pos.z = -6.8f;
						SuperController.singleton.monitorUIYOffset = 0f;
					}
					else if (Screen.height == 1600) // 2400x1600
					{
						pos.x = -1.95f;
						pos.y = 1.29f;
						pos.z = -6.25f;
						SuperController.singleton.monitorUIYOffset = 5f;
					}
					else if (Screen.height == 2000) // 3000x2000
					{
						pos.x = -3.20f;
						pos.y = 2.13f;
						pos.z = -4.8f;
						SuperController.singleton.monitorUIYOffset = 21f;
					}
					else if (Screen.height == 2160) // 3240x2160
					{
						pos.x = -3.68f;
						pos.y = 2.45f;
						pos.z = -4.25f;
						SuperController.singleton.monitorUIYOffset = 30f;
					}
					else
					{
						SuperController.singleton.monitorUIYOffset = 0f;
						SuperController.LogMessage("SallyUIscale4K WARNING - Untested unsupported resolution.");
					}
				}
				else if (aspectRatio <= 1.6001f) // 16:10
				{
					//pos.x = -3.925f; // y = 2.45f - z = -4.25f
					if (Screen.height == 1440) // 2304x1440
					{
						pos.x = -1.15f;
						pos.y = 0.72f;
						pos.z = -7.25f;
						SuperController.singleton.monitorUIYOffset = 0f;
					}
					else if (Screen.height == 1600) // 2560x1600
					{
						pos.x = -2.075f;
						pos.y = 1.29f;
						pos.z = -6.25f;
						SuperController.singleton.monitorUIYOffset = 5f;
					}
					else if (Screen.height == 1800) // 2880x1800
					{
						pos.x = -2.26f;
						pos.y = 1.41f;
						pos.z = -6.05f;
						SuperController.singleton.monitorUIYOffset = 9f;
					}
					else if (Screen.height == 1920) // 3072x1920
					{
						pos.x = -3.23f;
						pos.y = 2.02f;
						pos.z = -5.0f;
						SuperController.singleton.monitorUIYOffset = 20f;
					}
					else if (Screen.height == 2160) // 3456x2160
					{
						pos.x = -3.925f;
						pos.y = 2.45f;
						pos.z = -4.25f;
						SuperController.singleton.monitorUIYOffset = 30f;
					}
					else
					{
						SuperController.singleton.monitorUIYOffset = 0f;
						SuperController.LogMessage("SallyUIscale4K WARNING - Untested unsupported resolution.");
					}
				}
				else if (aspectRatio <= 1.78f)   // 16:9
				{
					if (Screen.height == 1440) // 2560x1440
					{
						pos.x = -1.28f;
						pos.y = 0.72f;
						pos.z = -7.25f;
						SuperController.singleton.monitorUIYOffset = 0f;
					}
					else if (Screen.height == 1800) // 3200x1800
					{
						pos.x = -3.12f;
						pos.y = 1.75f;
						pos.z = -5.45f;
						SuperController.singleton.monitorUIYOffset = 12f;
					}
					else if (Screen.height == 2160) // 3840x2160
					{
						pos.x = -4.36f;
						pos.y = 2.45f;
						pos.z = -4.25f;
						SuperController.singleton.monitorUIYOffset = 30f;
					}
					else
					{
						SuperController.singleton.monitorUIYOffset = 0f;
						SuperController.LogMessage("SallyUIscale4K WARNING - Untested unsupported resolution.");
					}
				}
				else if (aspectRatio <= 2.3889f) // ~21:9
				{

					if (Screen.height == 1440) // 3440x1440
					{
						pos.x = -3.44f;
						pos.y = 1.44f;
						pos.z = -6.00f;
						SuperController.singleton.monitorUIYOffset = 8f;
					}
					else
					{
						pos.x = -5.86f;
						pos.y = 2.45f;
						pos.z = -4.25f;
						SuperController.singleton.monitorUIYOffset = 0f;
						SuperController.LogMessage("SallyUIscale4K WARNING - Untested unsupported resolution.");
					}
				}
				else if (aspectRatio <= 2.4001f) // ~21:9
				{

					if (Screen.height == 1600) // 3840x1600
					{
						pos.x = -3.74f;
						pos.y = 1.55f;
						pos.z = -5.8f;
						SuperController.singleton.monitorUIYOffset = 8f;
					}
					else
					{
						pos.x = -5.885f;
						pos.y = 2.45f;
						pos.z = -4.25f;
						SuperController.singleton.monitorUIYOffset = 0f;
						SuperController.LogMessage("SallyUIscale4K WARNING - Untested unsupported resolution.");
					}
				}
				else if (aspectRatio <= 3.5556f) // 32:9
				{
					//pos.x = -6.54f; // y = 2.45f - z = -4.25f
					// 3840x1080, 
					pos.x = -4.105f;
					pos.y = 1.15f;
					pos.z = -6.5f;
					SuperController.singleton.monitorUIYOffset = 0f;
					SuperController.LogMessage("SallyUIscale4K WARNING - Untested unsupported resolution.");
					// 5120x1440(buggy on 4K monitor!)
				}
				else
					SuperController.LogError("SallyUIscale4K ERROR - Unsupported aspect ratio");
			}
			SuperController.singleton.MonitorUI.position = pos;

			// Update Sliders
			jfloatX.val = pos.x;
			jfloatY.val = pos.y;
			jfloatZ.val = pos.z;
			jfloatO.val = SuperController.singleton.monitorUIYOffset;
			UpdateCurrentText();
		}

		private void Deactivate()
        {
			SuperController.singleton.fixedMonitorUIScale = 1f;
			SuperController.singleton.monitorUIScale = 1f;
			SuperController.singleton.monitorUIYOffset = 0f;
			Vector3 pos;
			pos.x =  0.0f; //    0f default values (1920x1080 UIscale 1)
			pos.y =  0.0f; //    0f
			pos.z = -8.5f; // -8.5f
			SuperController.singleton.MonitorUI.position = pos;
			UpdateCurrentText();
		}

		private void PickRes(string s)
        {
			String[] res = s.Split('x');
			try
			{
				newWidth = Int32.Parse(res[0]);
				newHeight = Int32.Parse(res[1]);
			}
			catch (FormatException)
			{
				SuperController.LogError("Unable to parse " + s);
			}
		}

		private void ApplyRes()
        {
			if (newHeight < MIN_ACTIVE_HEIGHT)
			{
				jbActive.SetVal(false);
				Deactivate();
				SuperController.LogMessage("SallyUIscale4K DEACTIVATED to prevent unreachable UI (resolution height is < " + MIN_ACTIVE_HEIGHT + ")");
			}
			Screen.SetResolution(newWidth, newHeight, Screen.fullScreen);
			aspectRatio = (float)newWidth / (float)newHeight;
			autoUIscale = newHeight / 2160f;
			UpdateCurrentText();
			if (jbActive.val)
				Activate();
		}

		private string cText;
		private void UpdateCurrentText()
		{
			cText = "";
			if (newHeight < MIN_ACTIVE_HEIGHT)
				cText += "<b><color=#FF0000>Deactivated! Resolution < " + MIN_ACTIVE_HEIGHT + "p!</color></b>\n";

			cText += "Resolution  " + newWidth + " x " + newHeight +
				"\nAspect ratio " + aspectRatio + " to 1";
			if (jbActive.val)
				cText += "\nAutomatic UIscale " + autoUIscale +
				"\nBar position " + pos.x + "  " + pos.y + "  " + pos.z;
			jstringCurrentInfo.val = cText;
		}

		private static string GetLowResByNativeAR()
        {
			float nativeAspectRatio;
			if (debugRes)
				nativeAspectRatio = (float)debugNatWidth / (float)debugNatHeight;
			else 
				nativeAspectRatio = (float)Display.main.systemWidth / (float)Display.main.systemHeight;

			if (nativeAspectRatio <= 1.2501f) // 5:4
				return "1280x1024";
			if (nativeAspectRatio <= 1.3334f) // 4:3
				return "1440x1080";
			if (nativeAspectRatio <= 1.5001f) // 3:2
				return "1920x1280";
			if (nativeAspectRatio <= 1.6001f) // 16:10
				return "1920x1200";
			if (nativeAspectRatio <= 1.78f)   // 16:9
				return "1920x1080";
			if (nativeAspectRatio <= 2.3889f) // ~21:9
				return "2560x1080";
			if (nativeAspectRatio <= 2.4001f) // ~21:9 (3840x1600)
				return "2592x1080";
			if (nativeAspectRatio <= 3.5556f) // 32:9 (5120x1440)
				return "3840x1080";

			return "1920x1080";
		}

		private void SavePluginDataJSON()
		{
            FileManagerSecure.CreateDirectory(SAVE_FOLDER);
			JSONClass jc = new JSONClass
			{
				{"monitorUIScale", SuperController.singleton.monitorUIScale.ToString()},
				{"monitorUIYOffset", SuperController.singleton.monitorUIYOffset.ToString()},
				{"MonitorUI.position.x", SuperController.singleton.MonitorUI.position.x.ToString()},
                {"MonitorUI.position.y", SuperController.singleton.MonitorUI.position.y.ToString()},
                {"MonitorUI.position.z", SuperController.singleton.MonitorUI.position.z.ToString()},
            };

			string path = SAVE_FOLDER + "\\" + SAVE_FILENAME + "." + SAVE_EXTENSION;
            this.SaveJSON(jc, path);
            SuperController.LogMessage("SallyUIscale4K saved " + path);
        }

        private bool LoadPluginDataJSON()
        {
            string path = SAVE_FOLDER + "\\" + SAVE_FILENAME + "." + SAVE_EXTENSION;
            if (!FileManagerSecure.FileExists(path))
				return false;
            JSONClass jc = this.LoadJSON(path).AsObject;
			if (jc == null)
				return false;

            autoUIscale = jc["monitorUIScale"].AsFloat; // V7+
			if (!jc.HasKey("monitorUIYOffset"))
			{
                SuperController.LogMessage("SallyUIscale4K only loaded UI scale - saved data is from old version");
                return false;
			}
            SuperController.singleton.monitorUIYOffset = jc["monitorUIYOffset"].AsFloat; // V8+
            pos.x = jc["MonitorUI.position.x"].AsFloat; // V8+
            pos.y = jc["MonitorUI.position.y"].AsFloat; // V8+
            pos.z = jc["MonitorUI.position.z"].AsFloat; // V8+
            return true;
        }

		private void DeletePluginDataJSON()
		{
            string path = SAVE_FOLDER + "\\" + SAVE_FILENAME + "." + SAVE_EXTENSION;
			FileManagerSecure.DeleteFile(path);
            SuperController.LogMessage("SallyUIscale4K deleted " + path);
        }

        protected void SetBarPositionX(JSONStorableFloat v)
		{
			pos.x = v.val;
			SuperController.singleton.MonitorUI.position = pos;
			UpdateCurrentText();
		}

		protected void SetBarPositionY(JSONStorableFloat v)
		{
			pos.y = v.val;
			SuperController.singleton.MonitorUI.position = pos;
			UpdateCurrentText();
		}

		protected void SetBarPositionZ(JSONStorableFloat v)
		{
			pos.z = v.val;
			SuperController.singleton.MonitorUI.position = pos;
			UpdateCurrentText();
		}

		protected void SetBarOffset(JSONStorableFloat v)
		{
			SuperController.singleton.monitorUIYOffset = v.val;
		}
	}
}